package sam.machine.engine ;      

import java.util.* ; 
import java.util.concurrent.* ; 
import sam.machine.lang.* ; 
import sam.util.* ; 

/**
  *  ActiveRule is the Callable agent for applying 
  *  a sam.machine.lang.Rule to a branch using staged 
  *  inference.
  *  extended by    ActiveStagedRule 
  *           and   ActiveExistentialRule (pending ...)
  *  @version 9/10/2021
   */
public class ActiveRule implements ActiveInference { 

   protected final Rule rule ;              // the interpreted rule
   protected final int index ;              // 1-based index of this rule 
   protected final Functor[][] bindings ;   // variable bindings
   protected InferenceManager manager ;          
   public static int SkolemNumber = 0 ;  // static indexing of Skolem eigenvariables
   protected  int A,    // width of antecedent
                  C,    // A + widest consequent
                  U,    // number of universal variables in the antecedent
                  V ;   // total number of variables
   protected Functor[] factor ;  // the Functor factors of the antecedent 0..(A-1)
   protected Cache cache ;   
   protected int existential ;
   /**
     *  Construct an ActiveRule which knows its
     *  autolog Rule and the Tree which it builds.
     */
   public ActiveRule(Rule r, InferenceManager mngr, int indx) { 
      this.rule = r ; 
      this.index = indx ; 
      this.manager = mngr ;        
      this.A = r.A ; 
      this.C = r.C ;    
      this.V = r.V ; 
      this.U = r.U ; // U == 0 means factual 
      this.existential = r.existential ;
      this.bindings =  new Functor[V][C] ; //  [0,V) x [0,C)
      factor = new Functor[A] ;
      for (int f=0 ; f<A ; f++)  
         factor[f] = this.rule.antecedent.elementAt(f) ;
      this.cache = new Cache() ; 
   }  

////////////////////////////////////////////////////////////////
/*  bindings table, variables down left, rule factors across top

     antecedent [0,A)   consequent [A,C)
     0  1  2  3 ... A-1 A  ... C-1 C     
  0                    |           |
  1                    |           |
  2         []         |    *      |   Universal variable 
  .                    |           |    bindings [0,U) x [0,A)
  .                    |           |
  U-1                  |           |
  ---------------------|-----------|
  U                    |           |   
  .       *            |    []     |   Existential variable
  .                    |           |     bindings [U,V) x [A,C)
  V-1------------------|-----------| 
*///////////////////////////////////////////////////////////////

   /*
    *  Clear bindings
     */
   public void clear() { 
      for (int a = 0 ; a < V ; a++)     // [0,V)
         for (int b = 0 ; b < C ; b++)  // [0,C)   
            bindings[a][b] = null ; // unbound
   }

   /**
     *  @return table value of variable/index.
      */
   protected Functor lookup(int var) {
      for(int k=0 ; k < this.C ; k++) 
         if (bindings[var][k] != null)  
            return bindings[var][k] ; 
      return null ; // would be eigenvariable
   }

   /**
     *  Intended to be overridden by
     *     ActiveExistentialRule (restricted action)
     *     ActiveStagedRule (staged saturation)
      */
   public Cache call() {
      // fill in action here
      return this.cache ; 
   } 

/*
Lots of work to do ...

*/

}

