package sam.machine.lang ; 

import java.util.* ; 
import java.io.* ; 
import sam.util.* ; 
import sam.antlr.autolog.* ;  // Compiler

/**
  *  AutoLog Theory 
  *  @version 2/8/2021 
  */
public class Theory extends Vector<Inference> { 
   // Inferences are indexed in AutoLogAnalyzer
   public static int TRUE_INDEX = 0 ;  // name[0],equals"true"), etc.
   public static int GOAL_INDEX = 1 ;   
   public static int FALSE_INDEX = 2 ; 
   /// constructed via AutoLogAnalyzer 
   public String[] name ;                     // print names of functors by index
   public int[] arity ;                       // arity of functors by index
   public byte[] type ;                       // types of functors by index
   public Vector<Pattern>[] keyPatterns ;     // by index, set by AutoLogAnalyzer
   public Vector<Pattern>[] rewriteKeyPatterns ; // added 2/8/2021
   ///
   protected int count ;  // set to 3 by AA to index name[3] .. etc.
   // public HashMap<String,String> hints ;      // OLD UNUSED so far
   // public boolean subfactual ;                // OLD UNUSED so far
   // public PredicateRecord predrec ;           // OLD UNUSED so far 

   
   // CONSTRUCTOR makes empty Vector<Inference>
   // Theory fields are constructed by AutoLogAnalyzer

   /**
     *  Is every rule deep indexical?
     *  @version 11/21/2021 
     */
   public boolean isIndexical() {
      Iterator it = this.iterator() ; 
      while(it.hasNext()) 
         if (!((Inference)(it.next())).isIndexical()) return false ;
      return true ;
   }

   /**
     *  Is every rule deep predicative?
     *  @version 11/21/2021 
     */
   public boolean isPredicative() {
      Iterator it = this.iterator() ; 
      while(it.hasNext()) 
         if (!((Inference)(it.next())).isPredicative()) return false ;
      return true ;
   }

   /**
     *  Is every rule deep byName?
     *  @version 11/21/2021 
     */
   public boolean byName() {
      Iterator it = this.iterator() ; 
      while(it.hasNext()) 
         if (!((Inference)(it.next())).byName()) return false ;
      return true ;
   }

   /**
     *  Show Theory Summary 
     *  Called by report(Theory) in AutoLogAnalyzer
     *  @version 11/21/2021 
     */
   public StringBuffer showTheorySummary() {
      String s3="   ",
             s6="      " ;
      StringBuffer sb = new StringBuffer() ;
      sb.append("Theory Summary \n") ;
      sb.append(s3+"theory structural summary \n") ; 
         sb.append(s6+"theory.isIndexical()="+this.isIndexical()+"\n") ;
         sb.append(s6+"theory.isPredicative()="+this.isPredicative()+"\n") ;
         sb.append(s6+"theory.byName()="+this.byName()+"\n") ;
      sb.append(s3+"inference forms structural summaries (Rule/Rewrite) \n") ; 
      Iterator it = this.iterator() ; 
      int N=1 ; 
      while(it.hasNext()) {
         Inference inference = (Inference)it.next() ; 
         sb.append( s6+N+" .isIndexical()="+inference.isIndexical() ) ;
         sb.append( " .isPredicative()="+inference.isPredicative() ) ;
         sb.append( " .byName()="+inference.byName()+"\n" ) ;
         N+=1 ;
      }          
      return sb ;
   }

   

   /** 
     *  String display of this theory
     */
   public String toString() { 
      StringBuffer sb = new StringBuffer() ; 
      Iterator inferences = this.iterator() ;
      while (inferences.hasNext()) 
         sb.append(((Inference)inferences.next()).toString() + "\n") ;
      return sb.toString() ; 
   }


   /**
     *  Easy translate String to a theory, for testing.
      */
   public static Theory translate(String s) { 
      try { 
         InputStream is = new ByteArrayInputStream(s.getBytes()) ;
         return sam.antlr.autolog.Compiler.translate(is) ; 
      } 
      catch(Exception xxx) { 
         System.out.println(xxx) ;  // say what happened
         return null ;  // test for != null to continue 
      }
   }

   /**
     *  Testing Theory.translate ...  
     *  How can we catch BAD input gracefully?
     *  DUNNO yet.
      */
   public static void main(String[] args) { 
      //String s = "X∧X=X.  p,q=>r." ; 
      String s = "p(f(X))) (a) => goal. p=q." ;  // experimenting with an exceptions 
      Theory theory=translate(s) ; 
      System.out.println(theory) ; 
   }
   
}


